package com.muyu.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.marketing.service.ActivityTeamInfoService;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo>
        implements ActivityTeamProductSkuInfoService {

    @Autowired
    private ProjectSkuCache projectSkuCache;
    @Autowired
    private @Lazy ActivityTeamInfoService activityTeamInfoService;
    @Autowired
    private ActivityTeamProductSkuInfoMapper activityTeamProductSkuInfoMapper;

    /**
     * 通过团购活动ID获取团购中最优惠的价格
     * @param teamId 团购ID
     * @return 优惠价格
     */
    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        // 优惠模型集合
        Optional<TeamProductDiscountPriceModel> discountPriceModelOptional = teamProductSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
                    // 根据商品ID和商品SKU查询商品信息
                    ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                    return TeamProductDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
                }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());

        if (discountPriceModelOptional.isEmpty()) {
            throw new ServiceException("团购活动下没有商品绑定");
        }
        return discountPriceModelOptional.get();
    }

    /**
     * 通过活动ID获取 剩余库存
     *
     * @param teamId 活动ID
     * @return 库存
     */
    @Override
    public TeamProductStockModel getStock(Long teamId) {
        //根据活动id查询团购商品SKU集合
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        //封装返回对象
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L, Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L, Long::sum))
                .build();
    }

    /**
     * 添加团购商品SKU
     *
     * @param activityTeamProductSkuAddModelList 团购商品SKU添加模型集合
     */
    @Override
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList) {
        //判空
        if (activityTeamProductSkuAddModelList == null || activityTeamProductSkuAddModelList.isEmpty()) {
            throw new ServiceException("入参添加商品规格模型，不可为空");
        }
        //批量保存
        this.saveBatch(
                activityTeamProductSkuAddModelList.stream().map(ActivityTeamProductSkuInfo::modelBuild).toList()
        );
    }

    /**
     * 根据团购ID获取团购商品SKU列表
     *
     * @param teamId
     * @return
     */
    @Override
    public List<ActivityProductSkuListModel> getTeamProductSkuInfoListByTeamId(Long teamId) {
        //根据活动id查询团购商品SKU集合
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        //封装返回对象
        ArrayList<ActivityProductSkuListModel> activityProductSkuList = new ArrayList<>();
        //遍历
        for (ActivityTeamProductSkuInfo activityTeamProductSkuInfo : activityTeamProductSkuList) {
            //添加到集合中
            activityProductSkuList.add(ActivityProductSkuListModel.builder()
                    .id(activityTeamProductSkuInfo.getId())
                    .productId(activityTeamProductSkuInfo.getProductId())
                    .sku(activityTeamProductSkuInfo.getProductSku())
                    .teamPrice(activityTeamProductSkuInfo.getTeamPrice())
                    .teamStock(activityTeamProductSkuInfo.getTeamStock())
                    .build());
        }
        return activityProductSkuList;
    }

    /**
     * 编辑团购商品SKU集合
     *
     * @param productSkuUpdList
     * @return
     */
    @Override
    public boolean editBatchActivityTeamProductSkuUpdModelList(List<ActivityTeamProductSkuUpdModel> productSkuUpdList) {
        //创建updateWrapper
        LambdaUpdateWrapper<ActivityTeamProductSkuInfo> updateWrapper = new LambdaUpdateWrapper<>();
        //遍历集合
        for (ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel : productSkuUpdList) {
            //赋值
            updateWrapper.set(ActivityTeamProductSkuInfo::getTeamPrice, activityTeamProductSkuUpdModel.getTeamPrice());
            updateWrapper.set(ActivityTeamProductSkuInfo::getTeamStock, activityTeamProductSkuUpdModel.getTeamStock());
            updateWrapper.eq(ActivityTeamProductSkuInfo::getProductId, activityTeamProductSkuUpdModel.getProductId());
        }
        //返回
        return this.update(updateWrapper);
    }

    /**
     * 商品规格的批量删除
     * @param id
     * @return
     */
    @Override
    public boolean deleteBatchByTeamIds(List<Long> id) {
        //创建updateWrapper
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        //根据id查询
        List<Long> productIdList = activityTeamProductSkuInfoMapper.selectList(lambdaQueryWrapper).stream().map(ActivityTeamProductSkuInfo::getProductId).toList();
        //判断集合是否为空
        if (productIdList == null && productIdList.isEmpty()) {
            throw new ServiceException("该团购下没有商品");
        }
        return this.removeByIds(productIdList);
    }


}
