package com.muyu.product.service.impl;


import java.util.List;
import java.util.Objects;


import com.github.pagehelper.PageInfo;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.AttributeInfo;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.RuleAttrInfo;
import com.muyu.product.domain.model.RuleAttrAddModel;
import com.muyu.product.domain.model.RuleInfoAddModel;
import com.muyu.product.domain.req.RuleAttrInfoReq;
import com.muyu.product.domain.req.RuleInfoEditReq;
import com.muyu.product.domain.req.RuleInfoQueryReq;
import com.muyu.product.domain.req.RuleInfoSaveReq;
import com.muyu.product.domain.resp.RuleGroupUpdResp;
import com.muyu.product.domain.resp.RuleInfoResp;
import com.muyu.product.service.ProjectInfoService;
import com.muyu.product.service.RuleAttrInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.domain.RuleInfo;
import com.muyu.product.service.RuleInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 商品规格Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class RuleInfoServiceImpl extends ServiceImpl<RuleInfoMapper, RuleInfo> implements RuleInfoService {

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;
    @Autowired
    private RuleInfoMapper ruleAttrInfoMapper;
    @Autowired
    private ProjectInfoService projectInfoService;
    @Autowired
    private RuleInfoMapper ruleInfoMapper;

    /**
     * 查询商品规格列表
     *
     * @param ruleInfo 商品规格
     * @return 商品规格
     */
    @Override
    public List<RuleInfo> list(RuleInfo ruleInfo) {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleInfo.getName())) {
            queryWrapper.like(RuleInfo::getName, ruleInfo.getName());
        }

        if (ObjUtils.notNull(ruleInfo.getStatus())) {
            queryWrapper.eq(RuleInfo::getStatus, ruleInfo.getStatus());
        }

        return list(queryWrapper);
    }

    /**
     * 添加规格信息
     *
     * @param ruleInfoAddModel 规格添加模型
     * @return 时候成功
     */
    @Override
    public boolean save(RuleInfoAddModel ruleInfoAddModel) {
        RuleInfo ruleInfo = RuleInfo.addModelBuild(ruleInfoAddModel, SecurityUtils::getUsername);
        boolean save = this.save(ruleInfo);
        ruleAttrInfoService.saveBatch(
                ruleInfoAddModel.getRuleAttrList().stream()
                        .map(ruleAttrAddModel -> RuleAttrInfo.addModelBuild(ruleAttrAddModel, ruleInfo::getId, SecurityUtils::getUsername))
                        .toList()
        );
        return save;
    }

    /**
     * 查询规格信息
     *
     * @param ruleInfoQueryReq 规格查询
     * @return 规格信息
     */
    @Override
    public TableDataInfo<RuleInfoResp> queryList(RuleInfoQueryReq ruleInfoQueryReq) {
        List<RuleInfo> list = this.list(RuleInfo.queryBuild(ruleInfoQueryReq));
        List<RuleInfoResp> ruleInfoRespList = list.stream()
                .map(ruleInfo -> RuleInfoResp.infoBuild(ruleInfo, ruleId -> {
                    LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.eq(RuleAttrInfo::getRuleId, ruleId);
                    return ruleAttrInfoService.list(queryWrapper).stream().map(RuleAttrAddModel::infoBuild).toList();
                }))
                .toList();
        boolean isPage = ruleInfoQueryReq.getParams().get("isPage") == null || Convert.toBool(ruleInfoQueryReq.getParams().get("isPage"), true);
        List<ProjectInfo> projectInfoList = projectInfoService.list();
        for (ProjectInfo projectInfo : projectInfoList) {
            for (RuleInfoResp ruleInfoResp : ruleInfoRespList) {
                if (Objects.equals(projectInfo.getId(), ruleInfoResp.getId())){
                    ruleInfoResp.setDis(true);
                }
            }
        }
        return TableDataInfo.<RuleInfoResp>builder()
                .rows(ruleInfoRespList)
                .total(isPage ? new PageInfo<>(list).getTotal() : 0)
                .build();
    }

    @Override
    public RuleInfoResp getUpdById(Long id) {
        RuleInfoResp getRuleById = ruleAttrInfoMapper.getRuleById(id);
        List<RuleAttrAddModel> ruleAttrList = getRuleById.getRuleAttrList();
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            String attrValue = ruleAttrAddModel.getAttrValue();
            String[] split = attrValue.split(",");
            ruleAttrAddModel.setValueList(List.of(split));
        }
        return getRuleById;

//        RuleInfo ruleInfo = this.getById(id);
//
//        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
//        queryWrapper.eq(RuleAttrInfo::getRuleId,id);
//        List<RuleAttrInfo> ruleAttrInfoList = ruleAttrInfoService.list(queryWrapper);
//        List<Long> longList = ruleAttrInfoList.stream()
//                .map(RuleAttrInfo::getRuleId)
//                .toList();
//        List<RuleAttrInfo> ruleAttrInfos = ruleAttrInfoService.listByIds(longList);
//        return RuleGroupUpdResp.builder()
//                .name(ruleInfo.getName())
//                .id(ruleInfo.getId())
//                .status(ruleInfo.getStatus())
//                .ruleAttrList(ruleAttrInfos)
//                .build();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(Long id, RuleAttrInfoReq ruleInfoEditReq) {
        //修改商品规格表
        ruleAttrInfoMapper.updRule(id,ruleInfoEditReq);
        //删除中间表
        ruleAttrInfoMapper.deleteRule(id);
        //添加规格详情表
        List<RuleAttrAddModel> ruleAttrList = ruleInfoEditReq.getRuleAttrList();
        //遍历
        String value="";
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            //获取集合
            List<String> valueList = ruleAttrAddModel.getValueList();
            //定义值

            //遍历
            for (String s : valueList) {
                System.out.println(s);
               value+=","+s;
            }
            //截取
            String substring = value.substring(1);
            System.out.println(substring);
            //添加
            ruleAttrInfoMapper.addRuleAttrModel(id,ruleAttrAddModel.getName(),substring);
            value="";
        }
//        ruleAttrInfoService.delete(id);
//        List<AttributeInfo> attributeInfoList = ruleInfoEditReq.getAttributeInfoList();
//        for (AttributeInfo attributeInfo : attributeInfoList) {
//            ruleAttrInfoService.insertBatch(id, attributeInfo.getId());
//        }

    }

    @Override
    public Result have(Long id) {
        Integer count = ruleAttrInfoMapper.have(id);
        if (count>0){
            return Result.error(400,"规格下有商品，无法删除");
        }
        return Result.success(count);
    }

    @Override
    public String getName(Long ruleId) {
        return ruleInfoMapper.getName(ruleId);
    }
}
